#
# dinvgauss -- evaluate Inverse Gaussian density function
# rinvgauss -- simulate from the Inverse Gaussian distribution
#

#
# The Inverse Gaussian distribution with parameters a and b has density
#
#      f(x) = a/(sqrt(2 pi) x^(3/2)) e^(-a^2/(2x) - b^2 x/2 + ab)
#
#      for x > 0, a > 0 and b > 0.  The mean and variance are E(X) = a/b
#      and Var(X) = a/b^3.
#
# Simulation is done by rejection sampling with a Gamma distribution
#

dinvgauss <- function (x, a=1, b=1, log=FALSE)
{
   if (a <= 0 || b <= 0) {
      stop("\nInvalid parameter values.\n")
   }
   if (sum(x <= 0)>0) {
      stop("\nInputs must be positive.\n")
   }
   ld <- log(a) - log(2*pi)/2 - 3*log(x)/2 - a^2/(2*x) - b^2*x/2 + a*b
   if (log) { return (ld) }
   exp(ld)
}

rinvgauss <- function(n, a=1, b=1)
{
   if (a <= 0 || b <= 0) {
      stop("\nInvalid parameter values.\n")
   }
   r <- numeric(n)
   lc <- lgamma(a*b/2)+a*b*(1+log(2)-2*log(a*b))/2+(1+a*b)*log(1+a*b)/2-
         (1+log(pi)+log(2))/2
   for (i in 1:n) {
      while (1) {
         g <- rgamma(1, shape=a*b/2, rate=b^2/2)
         e <- rexp(1)
         if (e >= lc + dgamma(g, shape=a*b/2, rate=b^2/2, log=TRUE) -
             dinvgauss(g, a, b, log=TRUE)) { r[i] <- g; break; }
      }
   }
   r
}
