#
# Source R functions
#

source("../transform.r")
source("../optimize.r"); source("../slice.r");
source("../saddle.r"); source("../sumstats_sp.r");
source("../mcvar.r"); source("../plots.r");

#
# Set-up
#

outsum <- "summary_ss"; outpd <- "mc_pd"; outcdf <- "mc_cdf";
outhf <- "mc_hf"; outmc <- "mc_ss";
ntheta <- 3; N <- 15000; nburnin <- 1000; al <- .05;
m <- 1e-6; M <- 1e6;
xlim.up <- 200
t <- seq(m, xlim.up, 2)
bch <- 1; nb <- 30;

#
# Read the data
#

dat.df <- read.table("TRANS1.DAT", header=FALSE)
                                                                                
t12 <- dat.df[,4] - dat.df[,2]
tmp <- dat.df[,6] - dat.df[,4]
t23 <- tmp[dat.df[,7] == 3]; c2 <- tmp[dat.df[,7] == 2];
                                                                                
ip <- numeric(); ip[1] <- ntheta;
ip[2] <- length(t12); ip[3] <- ip[2]+length(t23);
ip[4] <- ip[3]+length(c2);
                                                                                
dp <- c(t12, t23, c2)

lb <- rep(0, ntheta)
ub <- rep(10, ntheta)
prior.p <- c(lb, ub)

dp <- c(dp, prior.p)

#
# Find MAP
#

MAP <- optimize(ntheta, ip, dp, maxiter=10000, outfile=outsum)

save.image()

#
# Slice sampling from posterior
#

Theta.ss <- slice(N, ntheta, ip, dp, start=MAP$theta, m=100, w=0.1,
                  alpha=al, nburn=nburnin, meth=2, outfile=outsum)
Theta.ss <- Theta.ss[[1]]

write.table(round(Theta.ss, 5), file="Theta.ss", quote=FALSE,
            row.names=FALSE, col.names=FALSE)

if (nburnin > 0) { Theta.ss.burnin <- Theta.ss[1:nburnin,] }
Theta.ss <- Theta.ss[(nburnin+1):(nburnin+N),]

save.image()

#
# Set Theta for Saddlepoint Approximation and Plots
#

Theta <- Theta.ss

#
# Saddlepoint approximation
#

SP <- saddle(N, Theta, t, m, M, nthread=3, file=TRUE)
pd <- SP$pd; nc <- SP$nc;
sp.lb <- SP$sp.lb; sp.ub <- SP$sp.ub;

save.image()

#
# Monte Carlo summaries
#

mc.pd <- read.table("pd.out") 
pd.mcsd <- sapply(1:length(t), function (i) { sqrt(mcvar(mc.pd[,i])$var) })

z.cv <- qnorm(0.975)
pd.lb <- pd - z.cv * pd.mcsd
pd.lb <- ifelse(pd.lb < 0, 0, pd.lb)
pd.ub <- pd + z.cv * pd.mcsd

write.table(cbind(pd.lb, pd, pd.ub), outpd, quote=FALSE,
            row.names=FALSE, col.names=FALSE)

save.image()

out <- sumstats(N, Theta, t, nc, sp.lb, sp.ub, numeric(length(t)),
                task=1, outfile=outsum)
mu <- out$vdk; var <- out$vddk + (mu - mean(mu))^2;

save.image()

out <- sumstats(N, Theta, t, nc, sp.lb, sp.ub, pd, task=2,
                nthread=3, file=TRUE)
cdf <- read.table("cdf.out")

save.image()

mu.mcmu <- mean(mu); var.mcmu <- mean(var+(mu-mu.mcmu)^2);
cdf.mcmu <- apply(cdf, 2, mean)

mu.mcsd <- sqrt(mcvar(mu)$var)
var.mcsd <- sqrt(mcvar(var+(mu-mu.mcmu)^2)$var)
cdf.mcsd <- sapply(1:length(t), function (i) { sqrt(mcvar(cdf[,i])$var) })

cdf.lb <- cdf.mcmu - z.cv * cdf.mcsd
cdf.lb <- ifelse(cdf.lb < 0, 0, cdf.lb)
cdf.ub <- cdf.mcmu + z.cv * cdf.mcsd
cdf.ub <- ifelse(cdf.ub > 1, 1, cdf.ub)

write.table(cbind(cdf.lb, cdf.mcmu, cdf.ub), outcdf, quote=FALSE,
            row.names=FALSE, col.names=FALSE)

save.image()

stats <- sumstats(N, Theta, t, nc, sp.lb, sp.ub, pd, cdf=cdf.mcmu,
                  task=3, nthread=3, outfile=outsum)
q <- stats$q
xlim.up <- max(xlim.up, q[length(q)])
                                                                                
save.image()
                                                                                
sf <- 1 - cdf.mcmu; hf <- pd / sf;
hf.est <- t(sapply(1:nb,
          function (i) {
             sb <- N/nb
             iss <- (1+(i-1)*sb):(i*sb)
             pd.t <- apply(mc.pd[iss,],2,mean)
             cdf.t <- apply(cdf[iss,],2,mean)
             pd.t / (1 - cdf.t)
          }))
hf.mcmu <- apply(hf.est, 2, mean)
hf.mcsd <- sapply(1:length(t),
                   function (i) {
                      if (bch) {
                         hf.mcvar <- mcvar(hf.est[,i], b=nb, method="batch")
                         c(hf.mcvar$acf, sqrt(hf.mcvar$var))
                      } else {
                         hf.mcvar <- mcvar(hf.est[,i])
                         c(hf.mcvar$ess, sqrt(hf.mcvar$var))
                      }
                   })
if (bch) { hf.acf <- hf.mcsd[1,] } else { hf.ess <- hf.mcsd[1,] }
hf.mcsd <- hf.mcsd[2,]

hf.lb <- hf.mcmu - z.cv * hf.mcsd
hf.lb <- ifelse(hf.lb < 0, 0, hf.lb)
hf.ub <- hf.mcmu + z.cv * hf.mcsd

write.table(cbind(hf.lb, hf.mcmu, hf, hf.ub), outhf, quote=FALSE,
            row.names=FALSE, col.names=FALSE)

save.image()

q.est <- t(sapply(1:nb,
         function (i) {
            sb <- N/nb
            iss <- (1+(i-1)*sb):(i*sb)
            cdf.t <- apply(cdf[iss,],2,mean)
            if (i == 1) {
               sumstats(sb, Theta[iss,], t, nc[iss], sp.lb[iss],
               sp.ub[iss], numeric(length(t)), cdf=cdf.t,
               task=3, nthread=3, vb=F, clean=F)$q
            } else if (i < nb) {
               sumstats(sb, Theta[iss,], t, nc[iss], sp.lb[iss],
               sp.ub[iss], numeric(length(t)), cdf=cdf.t,
               task=3, nthread=3, vb=F, compile=F, clean=F)$q
            } else {
               sumstats(sb, Theta[iss,], t, nc[iss], sp.lb[iss],
               sp.ub[iss], numeric(length(t)), cdf=cdf.t,
               task=3, nthread=3, vb=F, compile=F)$q
            }
         }))
rm(mc.pd, cdf)

save.image()

q.mcmu <- apply(q.est, 2, mean)
q.mcsd <- sapply(1:length(q),
                  function (i) {
                     if (bch) {
                        q.mcvar <- mcvar(q.est[,i], b=nb, method="batch")
                        c(q.mcvar$acf, sqrt(q.mcvar$var))
                     } else {
                        q.mcvar <- mcvar(q.est[,i])
                        c(q.mcvar$ess, sqrt(q.mcvar$var))
                     }
                  })
if (bch) { q.acf <- q.mcsd[1,] } else { q.ess <- q.mcsd[1,] }
q.mcsd <- q.mcsd[2,]

q.lb <- q.mcmu - z.cv * q.mcsd
q.lb <- ifelse(q.lb < 0, 0, q.lb)
q.ub <- q.mcmu + z.cv * q.mcsd

save.image()

write("Confidence Interval for PD Mean:\n", file=outmc)
write(c(mu.mcmu - z.cv * mu.mcsd, mu.mcmu, mu.mcmu + z.cv * mu.mcsd),
      file=outmc, ncol=3, append=TRUE)
write("\nConfidence Interval for PD Standard Deviation:\n",
      file=outmc, append=TRUE)
write(c(sqrt(var.mcmu - z.cv * var.mcsd), sqrt(var.mcmu),
        sqrt(var.mcmu + z.cv * var.mcsd)), file=outmc,
      ncol=3, append=TRUE)
write("\nConfidence Intervals for PD Quantiles:\n", file=outmc,
      append=TRUE)
write(t(cbind(q.lb, q.mcmu, q, q.ub)), file=outmc, ncol=4, append=TRUE)

#
# Plots
#

ps.options(width=6, height=6)
plots(t, pd, xlab="t", ylab="density", ulim=xlim.up,
      lwd=2, plot="pd")
plots(t, sf=sf, xlab="t", ylab="cdf", ulim=xlim.up,
      lwd=2, plot="cdf")
plots(t, hf=hf, xlab="t", ylab="hazard", ulim=xlim.up,
      lwd=2, plot="hf")
ps.options(width=0, height=0)
plots(theta=Theta, plot.mode=MAP$theta, plot="theta")
plots(theta=Theta, theta.burnin=Theta.ss.burnin, plot.mode=MAP$theta,
      plot="conv")
